import datetime
import os
import traceback
import requests
from instabuy_integration_utils.config import config


class IBException:
    @staticmethod
    def __save_local_log(data: str):
        with open(
            os.path.join(config.program_path, "log_errors.txt"), "a+", encoding="utf-8"
        ) as my_file:
            my_file.write("\n\n\n" + "".ljust(70, "*") + "\n")
            my_file.write(data)
            my_file.write("\n" + "".ljust(70, "*") + "\n")

    @staticmethod
    def save_exception(exception):

        try:
            robot_name = config.robot_name
        except AttributeError:
            robot_name = "generic"

        data = {
            "error": str(exception),
            "created_at": datetime.datetime.utcnow(),
            "robot": robot_name,
            "version": config.get_local_version(),
            "traceback": traceback.format_exception(
                exception, value=exception, tb=exception.__traceback__
            ),
        }

        response = requests.post(f"{config.api_url}/api_py_robots/errors", data=data)
        if response.status_code != 200:
            IBException.__save_local_log(str(data))



class IBAddress:
    def __init__(self, address_dict: dict):
        self.zipcode: str = address_dict["zipcode"]
        self.state: str = address_dict["state"]
        self.city: str = address_dict["city"]
        self.neighborhood: str = address_dict["neighborhood"]
        self.street: str = address_dict["street"]
        self.street_number: str = address_dict["street_number"]
        self.complement: str = address_dict["complement"]
        self.country: str = address_dict["country"]



import json
import os
import sys


class __Config:
    def __init__(self):
        self.api_url = "https://api.instabuy.com.br"
        self.items_batch_count = 1000
        self.program_path: str = os.path.dirname(os.path.abspath(sys.argv[0]))
        self.ib_integration_temp_files = os.path.join(self.program_path, "tmp")
        self.version_file_name: str = "version.txt"
        self.DEBUG = False

    def get_local_version(self):
        local_vpath = os.path.join(self.program_path, self.version_file_name)

        if not os.path.exists(local_vpath):
            return None

        local_version = ""
        try:
            with open(local_vpath, "r", encoding="utf-8") as local_vfile:
                local_version = local_vfile.read()
        except Exception as exception:
            print(exception)

        return local_version

    def load_local_config(self, local_file_name="config.json"):
        with open(os.path.join(self.program_path, local_file_name), "r") as local_file:
            local_config = json.load(local_file)

            for key in local_config:
                self.__setattr__(key, local_config[key])

    def __nested_dict_keys(self, dict_to_nest: dict) -> list:
        nested_dict_keys = []
        for key, value in dict_to_nest.items():
            if isinstance(value, dict):
                nested_dict_keys.append(key)
                nested_dict_keys.extend(self.__nested_dict_keys(value))
            else:
                nested_dict_keys.append(key)
        return nested_dict_keys

    def verify_local_config(
        self,
        local_file_name="config.json",
        config_template_file_name="config_template.json",
    ):
        local_config_path = os.path.join(self.program_path, local_file_name)
        config_template_path = os.path.join(
            self.program_path, config_template_file_name
        )

        with open(config_template_path, "r") as config_template_file:
            config_template = json.load(config_template_file)

        with open(local_config_path, "r") as local_config_file:
            local_config = json.load(local_config_file)

        config_keys = self.__nested_dict_keys(config_template)
        local_config_keys = self.__nested_dict_keys(local_config)
        if set(config_keys) != set(local_config_keys):
            print("Config file is invalid")
            return False

        return True


config = __Config()



from typing import List


class IBOrderProduct:
    def __init__(self, product_dict: dict = None):
        if product_dict:
            self.id: str = product_dict["id"]
            self.model_internal_code: str = product_dict["model_internal_code"]
            self.name: str = product_dict["name"]
            self.price: float = product_dict["price"]
            self.qtd: float = product_dict["qtd"]
            self.unit_type: str = product_dict["unit_type"]
            self.bar_codes: List[str] = product_dict["bar_codes"]
